==> v1.2020.1.tar.gz: plantuml/src/net/sourceforge/plantuml/ugraphic/comp/CompressionTransform.java <==
/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  http://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * http://plantuml.com/patreon (only 1$ per month!)
 * http://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 *
 * Original Author:  Arnaud Roques
 * 
 *
 */
package net.sourceforge.plantuml.ugraphic.comp;

import java.util.List;

public class CompressionTransform {

	private final List<Slot> all;

	public CompressionTransform(SlotSet slotSet) {
		this.all = slotSet.getSlots();
	}

	public double transform(double v) {
		return v - getCompressDelta(v);
	}

	private double getCompressDelta(double v) {
		double result = 0;
		for (Slot s : all) {
			if (s.getStart() > v) {
				continue;
			}
			if (v > s.getEnd()) {
				result += s.size();
			} else {
				result += v - s.getStart();
			}
		}
		return result;
	}

}

==> v1.2021.4.tar.gz: plantuml/src/net/sourceforge/plantuml/ugraphic/comp/CompressionTransform.java <==
/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  http://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * http://plantuml.com/patreon (only 1$ per month!)
 * http://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 *
 * Original Author:  Arnaud Roques
 * 
 *
 */
package net.sourceforge.plantuml.ugraphic.comp;

import java.util.List;

public class CompressionTransform implements PiecewiseAffineTransform {

	private final List<Slot> all;

	public CompressionTransform(SlotSet slotSet) {
		this.all = slotSet.getSlots();
	}

	public double transform(double v) {
		return v - getCompressDelta(v);
	}

	private double getCompressDelta(double v) {
		double result = 0;
		for (Slot s : all) {
			if (s.getStart() > v) {
				continue;
			}
			if (v > s.getEnd()) {
				result += s.size();
			} else {
				result += v - s.getStart();
			}
		}
		return result;
	}

}
